namespace Jsl.FxCop
{
    using Microsoft.FxCop.Sdk;
    using Microsoft.VisualStudio.TestTools.UnitTesting;

    [TestClass]
    public class TestDoNotUseToStringOnAString
    {
        [TestMethod]
        public void ToString_against_a_string_produces_problem()
        {
            new DoNotUseToStringOnAString()
                .RunMethodRule(
                    () =>
                    {
                        string x = "123";
                        string y = x.ToString();
                        string z = y;           // Needed so that the y variable doesn't get optimized out.
                    })
                .HasProblem(
                    100,
                    FixCategories.NonBreaking,
                    MessageLevel.Warning,
                    "Remove the call to ToString since the object is already a string.")
                .HasNoMoreProblems();
        }

        [TestMethod]
        public void ToString_not_against_a_string_produces_no_problems()
        {
            new DoNotUseToStringOnAString()
                .RunMethodRule(
                    () =>
                    {
                        string x = this.ToString();
                        string y = x;           // Needed so that the x variable doesn't get optimized out.
                    })
                .HasNoMoreProblems();
        }
    }
}
